package ch.fritteli.labyrinth.generator;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.net.TheListener;
import ch.fritteli.labyrinth.generator.renderer.html.HTMLRenderer;
import ch.fritteli.labyrinth.generator.renderer.htmlfile.HTMLFileRenderer;
import ch.fritteli.labyrinth.generator.renderer.pdffile.PDFFileRenderer;
import ch.fritteli.labyrinth.generator.renderer.text.TextRenderer;
import ch.fritteli.labyrinth.generator.renderer.textfile.TextFileRenderer;
import io.vavr.control.Option;
import lombok.NonNull;

import java.nio.file.Path;
import java.nio.file.Paths;

public class Main {
    public static void main(@NonNull final String[] args) {
        final Option<TheListener> listener = TheListener.createListener();
        if (listener.isEmpty()) {
            System.out.println("In order to start the webserver, specify the port to listen to via the system property 'fritteli.labyrinth.listenerport', i.e.: -Dfritteli.labyrinth.listenerport=12345");
            int width = 100;
            int height = 100;
            final Labyrinth labyrinth = new Labyrinth(width, height/*, 0*/);
            final TextRenderer textRenderer = TextRenderer.newInstance();
            final HTMLRenderer htmlRenderer = HTMLRenderer.newInstance();
            final Path userHome = Paths.get(System.getProperty("user.home"));
            final String baseFilename = getBaseFilename(labyrinth);
            final TextFileRenderer textFileRenderer = TextFileRenderer.newInstance()
                    .setTargetLabyrinthFile(userHome.resolve(baseFilename + ".txt"))
                    .setTargetSolutionFile(userHome.resolve(baseFilename + "-solution.txt"));
            final HTMLFileRenderer htmlFileRenderer = HTMLFileRenderer.newInstance()
                    .setTargetFile(userHome.resolve(baseFilename + ".html"));
            final PDFFileRenderer pdfFileRenderer = PDFFileRenderer.newInstance()
                    .setTargetFile(userHome.resolve(baseFilename + ".pdf"));

            System.out.println("Labyrinth-ID: " + labyrinth.getRandomSeed());
            // Render Labyrinth to stdout
            System.out.println(textRenderer.render(labyrinth));
            // Render Labyrinth solution to stdout
            System.out.println(textRenderer.setRenderSolution(true).render(labyrinth));
            // Render HTML to stdout
            System.out.println(htmlRenderer.render(labyrinth));
            // Render Labyrinth and solution to (separate) files
            System.out.println(textFileRenderer.render(labyrinth));
            // Render HTML to file
            System.out.println(htmlFileRenderer.render(labyrinth));
            // Render PDF to file
            System.out.println(pdfFileRenderer.render(labyrinth));
        }
    }

    private static String getBaseFilename(@NonNull final Labyrinth labyrinth) {
        return "labyrinth-" + labyrinth.getWidth() + "x" + labyrinth.getHeight() + "-" + labyrinth.getRandomSeed();
    }
}
