package ch.fritteli.labyrinth.generator.renderer.html;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.Renderer;
import lombok.NonNull;

public class HTMLRenderer implements Renderer<String> {
    private static final String POSTAMBLE = "</body></html>";

    private HTMLRenderer() {
    }

    @NonNull
    public static HTMLRenderer newInstance() {
        return new HTMLRenderer();
    }

    @NonNull
    public String render(@NonNull final Labyrinth labyrinth) {
        if (labyrinth.getWidth() == 0 || labyrinth.getHeight() == 0) {
            return this.getPreamble(labyrinth) + POSTAMBLE;
        }
        final Generator generator = new Generator(labyrinth);
        final StringBuilder sb = new StringBuilder(this.getPreamble(labyrinth));
        sb.append("<table>");
        while (generator.hasNext()) {
            sb.append(generator.next());
        }
        sb.append("</table>");
        sb.append(POSTAMBLE);
        return sb.toString();
    }

    private String getPreamble(@NonNull final Labyrinth labyrinth) {
        return "<!DOCTYPE html><html lang=\"en\">" +
                "<head>" +
                "<title>Labyrinth " + labyrinth.getWidth() + "x" + labyrinth.getHeight() + ", ID " + labyrinth.getRandomSeed() + "</title>" +
                "<meta charset=\"utf-8\">" +
                "<style>" +
                "table{border-collapse:collapse;}" +
                "td{border:0 solid black;height:1em;width:1em;}" +
                "td.top{border-top-width:1px;}" +
                "td.right{border-right-width:1px;}" +
                "td.bottom{border-bottom-width:1px;}" +
                "td.left{border-left-width:1px;}" +
                "</style>" +
                "<script>" +
                "let solution = false;" +
                "function toggleSolution() {" +
                "let stylesheet = document.styleSheets[0];" +
                "if(solution){" +
                "stylesheet.deleteRule(0);" +
                "}else{" +
                "stylesheet.insertRule(\"td.solution{background-color:lightgray;}\", 0);" +
                "}" +
                "solution = !solution;" +
                "}" +
                "</script>" +
                "</head>" +
                "<body>" +
                "<input type=\"checkbox\" onclick=\"toggleSolution()\">show solution</input>";
    }

}
