package ch.fritteli.labyrinth.generator.renderer.htmlfile;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.Renderer;
import ch.fritteli.labyrinth.generator.renderer.html.HTMLRenderer;
import lombok.NonNull;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;

public class HTMLFileRenderer implements Renderer<Path> {
    @NonNull
    private static final HTMLRenderer HTML_RENDERER = HTMLRenderer.newInstance();
    private Path targetFile;

    private HTMLFileRenderer() {
        try {
            this.targetFile = Files.createTempFile("labyrinth_", ".html");
        } catch (IOException e) {
            System.err.println("Unable to set default target file.");
            e.printStackTrace();
        }
    }

    @NonNull
    public static HTMLFileRenderer newInstance() {
        return new HTMLFileRenderer();
    }

    public boolean isTargetFileDefinedAndWritable() {
        return this.targetFile != null && this.targetFile.toFile().canWrite();
    }

    @NonNull
    public HTMLFileRenderer setTargetFile(@NonNull final Path targetFile) {
        this.targetFile = targetFile;
        return this;
    }

    @Override
    public Path render(@NonNull final Labyrinth labyrinth) {
        if (!this.isTargetFileDefinedAndWritable()) {
            try {
                Files.createFile(this.targetFile);
            } catch (IOException e) {
                throw new IllegalArgumentException("Cannot write to target file.", e);
            }
        }
        final String html = HTML_RENDERER.render(labyrinth);
        try {
            Files.writeString(this.targetFile, html, StandardCharsets.UTF_8);
        } catch (IOException e) {
            System.err.println("Failed writing to file " + this.targetFile.normalize().toString());
            e.printStackTrace();
        }
        return targetFile;
    }
}
