package ch.fritteli.labyrinth.generator.renderer.pdffile;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.Renderer;
import ch.fritteli.labyrinth.generator.renderer.pdf.PDFRenderer;
import lombok.NonNull;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

public class PDFFileRenderer implements Renderer<Path> {
    @NonNull
    private static final PDFRenderer PDF_RENDERER = PDFRenderer.newInstance();
    private Path targetFile;

    private PDFFileRenderer() {
        try {
            this.targetFile = Files.createTempFile("labyrinth_", ".pdf");
        } catch (IOException e) {
            System.err.println("Unable to set default target file.");
            e.printStackTrace();
        }
    }

    @NonNull
    public static PDFFileRenderer newInstance() {
        return new PDFFileRenderer();
    }

    public boolean isTargetFileDefinedAndWritable() {
        return this.targetFile != null && this.targetFile.toFile().canWrite();
    }

    @NonNull
    public PDFFileRenderer setTargetFile(@NonNull final Path targetFile) {
        this.targetFile = targetFile;
        return this;
    }

    @Override
    public Path render(@NonNull final Labyrinth labyrinth) {
        if (!this.isTargetFileDefinedAndWritable()) {
            try {
                Files.createFile(this.targetFile);
            } catch (IOException e) {
                throw new IllegalArgumentException("Cannot write to target file.", e);
            }
        }
        final byte[] bytes = PDF_RENDERER.render(labyrinth);
        try {
            Files.write(this.targetFile, bytes);
        } catch (IOException e) {
            System.err.println("Failed writing to file " + this.targetFile.normalize().toString());
            e.printStackTrace();
        }
        return this.targetFile;
    }
}
