package ch.fritteli.labyrinth.generator.model;

import io.vavr.collection.Stream;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

class WallsTest {
    @Test
    void testConstruct() {
        // arrange / act
        final Walls sut = new Walls();

        // assert
        assertFalse(sut.isSet(Direction.TOP));
        assertFalse(sut.isSet(Direction.RIGHT));
        assertFalse(sut.isSet(Direction.BOTTOM));
        assertFalse(sut.isSet(Direction.LEFT));
    }

    @Test
    void testSet() {
        // arrange
        final Walls sut = new Walls();

        //act
        sut.set(Direction.RIGHT);

        // assert
        assertFalse(sut.isSet(Direction.TOP));
        assertTrue(sut.isSet(Direction.RIGHT));
        assertFalse(sut.isSet(Direction.BOTTOM));
        assertFalse(sut.isSet(Direction.LEFT));

        //act: Setting twice has no effect
        sut.set(Direction.RIGHT);

        // assert
        assertFalse(sut.isSet(Direction.TOP));
        assertTrue(sut.isSet(Direction.RIGHT));
        assertFalse(sut.isSet(Direction.BOTTOM));
        assertFalse(sut.isSet(Direction.LEFT));
    }

    @Test
    void testSetAll() {
        // arrange
        final Walls sut = new Walls();

        //act
        sut.setAll();

        // assert
        assertTrue(sut.isSet(Direction.TOP));
        assertTrue(sut.isSet(Direction.RIGHT));
        assertTrue(sut.isSet(Direction.BOTTOM));
        assertTrue(sut.isSet(Direction.LEFT));
    }

    @Test
    void testHarden() {
        // arrange
        final Walls sut = new Walls();
        sut.set(Direction.TOP);
        sut.set(Direction.RIGHT);

        // act
        sut.harden(Direction.TOP);

        // assert
        assertTrue(sut.isSet(Direction.TOP));
        assertTrue(sut.isSet(Direction.RIGHT));
        assertFalse(sut.isSet(Direction.BOTTOM));
        assertFalse(sut.isSet(Direction.LEFT));

        // act: try to clear hardened wall
        final boolean result = sut.clear(Direction.TOP);

        // assert: TOP wall is still set
        assertFalse(result);
        assertTrue(sut.isSet(Direction.TOP));

        // act / assert: try to harden un-set wall
        assertThrows(IllegalStateException.class, () -> sut.harden(Direction.LEFT));
    }

    @Test
    void testUnharden() {
        // arrange
        final Walls sut = new Walls();
        sut.setAll();
        sut.harden(Direction.TOP);

        // pre-assert: TOP can't be cleared while hardened
        assertFalse(sut.clear(Direction.TOP));

        // act
        sut.unharden(Direction.TOP);

        // assert: TOP can be cleared
        assertTrue(sut.clear(Direction.TOP));
    }

    @Test
    void testGetUnhardenedSet() {
        // arrange
        final Walls sut = new Walls();

        // act
        Stream<Direction> result = sut.getUnhardenedSet();

        // assert
        assertTrue(result.isEmpty());

        // arrange: set some directions
        sut.set(Direction.TOP);
        sut.set(Direction.LEFT);

        // act
        result = sut.getUnhardenedSet();

        // assert
        assertEquals(Stream.of(Direction.TOP, Direction.LEFT), result);

        // arrange: harden a direction
        sut.harden(Direction.TOP);

        // act
        result = sut.getUnhardenedSet();

        // assert
        assertEquals(Stream.of(Direction.LEFT), result);
    }
}
